;
;	
;	
;			Title:	AVR RS232 interface 	
;			Date:	14.09.2003
;			Target:	AT90s2313 4MHz
;			Author:	Serasidis Vasilis
;			Home:	Http://www.serasidis.gr
;			eMail:	info@serasidis.gr
;	
;			Description:
;			This example shows how to control some AVR device via Visual Basic.
;			The hardware constituted by AT90s2313 at 4MHz speed, MAX232 or similar,
;			8 LEDs connected to PortB and 5 switches connected to PortD. 
;			Alternative	you can use some STK board like MCU 00100, STK200, STK500.

;
;			The software:
;			When you press some button from Visual Basic (VB) software you will see the LEDs
;			on the board, to change there status (on-off). When you press the switches from the 
;			board you will see the color of the pins PD2-PD6, in the VB software, to change there color
;			from green to red.



.include "2313def.inc"		;Define chip particulars
.list

.equ	BAUD	=24 		;9600 bps at 4 MHz.

.def	temp	=R16		;temporary storage register
.def	temp2	=R17
.def	EEard	=R18		;EEPROM address to read from
.def	txbyte	=R19		;Data to be transmitted
.def	rxbyte	=R20		;Received data
.def	delayR	=R21
.def	txtemp	=r22
.def	rxtemp	=r23

.cseg

.org	0


		ldi		temp,RAMEND
		out		SPL,temp	;Init Stack Pointer


		ldi     temp, 0b00000000        ;set port D as inputs
		out     DDRD, temp
		ldi     temp, 0b11111111        ;enable pull-up
		out     PortD, temp
		ldi     temp, 0b11111111        ;set port B as outputs
		out     DDRB, temp

		ldi		temp, BAUD
		out		UBRR, temp	;Set baud rate generator

;===========   Set UART, as Transmiter   =====================
	
		ldi		temp,0b00011000
		out		UCR,temp	;Enable UART Tx w/o interrupts
;==============================================================

		
load:
		rcall	delay			;Delay some time
		rcall	delay			;Delay some time			
		in		txbyte,pinD		;read the port D
		lsr		txbyte			;shift the txbyte register right
		lsr		txbyte			;shift one more time the txbyte register right
		cp		temp,txbyte		; compare temp with txbyte
		breq	cont			; if are the same go to cont

		cp		txbyte,temp		;$1f is when release the key
		breq	cont
		rcall	b_transmit

cont:

		in		temp,pinD		;Read portD...
		lsr		temp
		lsr		temp

		rjmp	receive			;Wait character from PC



;***************************************************************************
; Transmiting routine (The data from the eeprom through the AVR are going to
; the PC computer)         
;***************************************************************************
b_transmit:	
		sbis	USR,UDRE	;is UART transmitter ready?
		rjmp	b_transmit
		out		UDR,txbyte	;sent out char
		ret

;*************************************************************************
; Receiving routine (The data from PC computer through to the AVR are going
; to the eeprom)  
;*************************************************************************
receive:
		sbis	USR,RXC
		rjmp	load
		in		rxbyte,UDR
		out		portb,rxbyte
		rjmp	load

;================= Delay routine ================
delay:		ldi		delay,255
again:		dec		delayR
			brne	again
			ret